==> orign-0.2.37-py3-none-any.whl: orign/buffers/buffer.py <==
from typing import Dict, List, Optional

import requests
from nebu import V1ContainerRequest, V1ResourceMetaRequest, V1ResourceReference

from orign.auth import get_user_profile
from orign.buffers.models import (
    V1ReplayBuffer,
    V1ReplayBufferData,
    V1ReplayBufferRequest,
    V1ReplayBuffersResponse,
    V1SampleBufferQuery,
    V1SampleResponse,
    V1UpdateReplayBufferRequest,
)
from orign.config import GlobalConfig


class ReplayBuffer:
    def __init__(
        self,
        name: str,
        train_job: Optional[V1ContainerRequest] = None,
        namespace: Optional[str] = None,
        train_every: int = 50,
        sample_n: int = 100,
        sample_strategy: str = "Random",
        labels: Optional[Dict[str, str]] = None,
        config: Optional[GlobalConfig] = None,
    ):
        config = config or GlobalConfig.read()
        current_server = config.get_current_server_config()
        if not current_server:
            raise ValueError("No current server config found.")
        self.api_key = current_server.api_key
        self.orign_host = current_server.server

        # Construct the WebSocket URL with query parameters
        self.buffers_url = f"{self.orign_host}/v1/buffers"

        name_parts = name.split("/")
        if len(name_parts) == 2:
            self.namespace = name_parts[0]
            self.name = name_parts[1]
        else:
            self.namespace = namespace
            self.name = name

        if not self.namespace:
            if not self.api_key:
                raise ValueError("No API key provided")

            user_profile = get_user_profile(self.api_key)
            self.namespace = user_profile.handle

            if not self.namespace:
                self.namespace = user_profile.email.replace("@", "-").replace(".", "-")

        response = requests.get(
            self.buffers_url, headers={"Authorization": f"Bearer {self.api_key}"}
        )
        response.raise_for_status()
        buffers = V1ReplayBuffersResponse.model_validate(response.json())
        print(buffers)
        self.buffer = next(
            (
                b
                for b in buffers.buffers
                if b.metadata.name == self.name
                and b.metadata.namespace == self.namespace
            ),
            None,
        )

        if not self.buffer:
            print(f"Creating buffer {self.name} in namespace {self.namespace}")
            request = V1ReplayBufferRequest(
                metadata=V1ResourceMetaRequest(
                    name=self.name,
                    namespace=self.namespace,
                    labels=labels,
                ),
                train_every=train_every,
                sample_n=sample_n,
                sample_strategy=sample_strategy,
                train_job=train_job,
            )

            response = requests.post(
                self.buffers_url,
                json=request.model_dump(),
                headers={"Authorization": f"Bearer {self.api_key}"},
            )
            response.raise_for_status()

            self.buffer = V1ReplayBuffer.model_validate(response.json())
            print(f"Created buffer {self.buffer.metadata.name}")

        else:
            print(f"Found buffer {self.buffer.metadata.name}, updating if necessary")
            request = V1UpdateReplayBufferRequest(
                train_every=train_every,
                sample_n=sample_n,
                sample_strategy=sample_strategy,
                train_job=train_job,
            )

            response = requests.patch(
                f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}",
                json=request.model_dump(),
                headers={"Authorization": f"Bearer {self.api_key}"},
            )

            response.raise_for_status()
            resp_parsed = response.json()
            print("response json: ", resp_parsed)

            if "buffer" in resp_parsed:
                self.buffer = V1ReplayBuffer.model_validate(resp_parsed["buffer"])
            else:
                self.buffer = V1ReplayBuffer.model_validate(resp_parsed)
            print(f"Updated buffer {self.buffer.metadata.name}")

    def send(self, data: List[dict], train: Optional[bool] = None):
        if not self.buffer or not self.buffer.metadata.name:
            raise ValueError("Buffer not found")

        url = f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}/examples"

        request = V1ReplayBufferData(examples=data, train=train)  # type: ignore

        response = requests.post(
            url,
            json=request.model_dump(),
            headers={"Authorization": f"Bearer {self.api_key}"},
        )
        response.raise_for_status()
        return response.json()

    def sample(
        self,
        n: int = 10,
        strategy: str = "Random",
        link: bool = False,
    ) -> V1SampleResponse:
        """
        Samples data from the replay buffer using a POST request.

        Args:
            sample_n: The number of samples to retrieve.
            sample_strategy: The sampling strategy to use (e.g., "Random").

        Returns:
            A V1SampleResponse object containing information about the sampled dataset.
        """
        if not self.buffer or not self.buffer.metadata.name:
            raise ValueError("Buffer not found")

        url = f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}/sample"
        query = V1SampleBufferQuery(n=n, strategy=strategy, link=link)

        response = requests.post(
            url,
            json=query.model_dump(),
            headers={"Authorization": f"Bearer {self.api_key}"},
        )
        response.raise_for_status()
        return V1SampleResponse.model_validate(response.json())

    def train(self):
        if not self.buffer or not self.buffer.metadata.name:
            raise ValueError("Buffer not found")

        url = f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}/train"
        response = requests.post(
            url, headers={"Authorization": f"Bearer {self.api_key}"}
        )
        response.raise_for_status()
        return response.json()

    @classmethod
    def get(
        cls,
        namespace: Optional[str] = None,
        name: Optional[str] = None,
        config: Optional[GlobalConfig] = None,
    ) -> List[V1ReplayBuffer]:
        config = config or GlobalConfig.read()
        current_server = config.get_current_server_config()
        if not current_server:
            raise ValueError("No current server config found.")

        # Construct the WebSocket URL with query parameters
        buffers_url = f"{current_server.server}/v1/buffers"

        response = requests.get(
            buffers_url, headers={"Authorization": f"Bearer {current_server.api_key}"}
        )
        response.raise_for_status()
        buffer_response = V1ReplayBuffersResponse.model_validate(response.json())
        buffers = buffer_response.buffers
        if name:
            buffers = [b for b in buffers if b.metadata.name == name]

        if namespace:
            buffers = [b for b in buffers if b.metadata.namespace == namespace]

        return buffers

    def ref(self) -> V1ResourceReference:
        if not self.buffer:
            raise ValueError("Buffer not found")
        return V1ResourceReference(
            name=self.buffer.metadata.name,
            namespace=self.buffer.metadata.namespace,
            kind="ReplayBuffer",
        )

==> orign-0.2.58-py3-none-any.whl: orign/buffers/buffer.py <==
from typing import Dict, List, Optional

import requests
from nebu import V1ResourceMetaRequest, V1ResourceReference

from orign.buffers.models import (
    V1ReplayBuffer,
    V1ReplayBufferData,
    V1ReplayBufferRequest,
    V1ReplayBuffersResponse,
    V1SampleBufferQuery,
    V1SampleResponse,
    V1Trainer,
    V1UpdateReplayBufferRequest,
)
from orign.config import GlobalConfig


class ReplayBuffer:
    def __init__(
        self,
        name: str,
        namespace: Optional[str] = None,
        trainer: Optional[V1Trainer] = None,
        train_every: Optional[int] = None,
        sample_n: Optional[int] = None,
        sample_strategy: Optional[str] = None,
        labels: Optional[Dict[str, str]] = None,
        config: Optional[GlobalConfig] = None,
    ):
        config = config or GlobalConfig.read()
        current_server = config.get_current_server_config()
        if not current_server:
            raise ValueError("No current server config found.")
        self.api_key = current_server.api_key
        self.orign_host = current_server.server

        # Construct the WebSocket URL with query parameters
        self.buffers_url = f"{self.orign_host}/v1/buffers"

        name_parts = name.split("/")
        if len(name_parts) == 2:
            self.namespace = name_parts[0]
            self.name = name_parts[1]
        else:
            self.namespace = namespace
            self.name = name

        if not self.namespace:
            self.namespace = "-"

        response = requests.get(
            self.buffers_url, headers={"Authorization": f"Bearer {self.api_key}"}
        )
        response.raise_for_status()
        buffers = V1ReplayBuffersResponse.model_validate(response.json())
        print(buffers)
        self.buffer = next(
            (
                b
                for b in buffers.buffers
                if b.metadata.name == self.name
                and b.metadata.namespace == self.namespace
            ),
            None,
        )

        if not self.buffer:
            print(f"Creating buffer {self.name} in namespace {self.namespace}")
            request = V1ReplayBufferRequest(
                metadata=V1ResourceMetaRequest(
                    name=self.name,
                    namespace=self.namespace,
                    labels=labels,
                ),
                train_every=train_every,
                sample_n=sample_n,
                sample_strategy=sample_strategy,
                trainer=trainer,
            )

            response = requests.post(
                self.buffers_url,
                json=request.model_dump(),
                headers={"Authorization": f"Bearer {self.api_key}"},
            )
            response.raise_for_status()

            self.buffer = V1ReplayBuffer.model_validate(response.json())
            print(f"Created buffer {self.buffer.metadata.name}")

        else:
            print(f"Found buffer {self.buffer.metadata.name}, updating if necessary")
            request = V1UpdateReplayBufferRequest(
                train_every=train_every,
                sample_n=sample_n,
                sample_strategy=sample_strategy,
                trainer=trainer,
            )

            response = requests.patch(
                f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}",
                json=request.model_dump(),
                headers={"Authorization": f"Bearer {self.api_key}"},
            )

            response.raise_for_status()
            resp_parsed = response.json()
            print("response json: ", resp_parsed)

            if "buffer" in resp_parsed:
                self.buffer = V1ReplayBuffer.model_validate(resp_parsed["buffer"])
            else:
                self.buffer = V1ReplayBuffer.model_validate(resp_parsed)
            print(f"Updated buffer {self.buffer.metadata.name}")

    def send(self, data: List[dict], train: Optional[bool] = None):
        if not self.buffer or not self.buffer.metadata.name:
            raise ValueError("Buffer not found")

        url = f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}/examples"

        request = V1ReplayBufferData(examples=data, train=train)  # type: ignore

        response = requests.post(
            url,
            json=request.model_dump(),
            headers={"Authorization": f"Bearer {self.api_key}"},
        )
        response.raise_for_status()
        return response.json()

    def sample(
        self,
        n: int = 10,
        strategy: str = "Random",
        link: bool = False,
    ) -> V1SampleResponse:
        """
        Samples data from the replay buffer using a POST request.

        Args:
            sample_n: The number of samples to retrieve.
            sample_strategy: The sampling strategy to use (e.g., "Random").

        Returns:
            A V1SampleResponse object containing information about the sampled dataset.
        """
        if not self.buffer or not self.buffer.metadata.name:
            raise ValueError("Buffer not found")

        url = f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}/sample"
        query = V1SampleBufferQuery(n=n, strategy=strategy, link=link)

        response = requests.post(
            url,
            json=query.model_dump(),
            headers={"Authorization": f"Bearer {self.api_key}"},
        )
        response.raise_for_status()
        return V1SampleResponse.model_validate(response.json())

    def train(self):
        if not self.buffer or not self.buffer.metadata.name:
            raise ValueError("Buffer not found")

        url = f"{self.buffers_url}/{self.buffer.metadata.namespace}/{self.buffer.metadata.name}/train"
        response = requests.post(
            url, headers={"Authorization": f"Bearer {self.api_key}"}
        )
        response.raise_for_status()
        return response.json()

    @classmethod
    def get(
        cls,
        namespace: Optional[str] = None,
        name: Optional[str] = None,
        config: Optional[GlobalConfig] = None,
    ) -> List[V1ReplayBuffer]:
        config = config or GlobalConfig.read()
        current_server = config.get_current_server_config()
        if not current_server:
            raise ValueError("No current server config found.")

        # Construct the WebSocket URL with query parameters
        buffers_url = f"{current_server.server}/v1/buffers"

        response = requests.get(
            buffers_url, headers={"Authorization": f"Bearer {current_server.api_key}"}
        )
        response.raise_for_status()
        buffer_response = V1ReplayBuffersResponse.model_validate(response.json())
        buffers = buffer_response.buffers
        if name:
            buffers = [b for b in buffers if b.metadata.name == name]

        if namespace:
            buffers = [b for b in buffers if b.metadata.namespace == namespace]

        return buffers

    def ref(self) -> V1ResourceReference:
        if not self.buffer:
            raise ValueError("Buffer not found")
        return V1ResourceReference(
            name=self.buffer.metadata.name,
            namespace=self.buffer.metadata.namespace,
            kind="ReplayBuffer",
        )
